# -*- coding: utf-8 -*-
from django.conf import settings

from django import forms
from django.db.models import Q, F, expressions, functions
from django.utils.translation import ugettext_lazy as _
from django.utils.six import with_metaclass

from job_offers.models import Application, RelatedObject


JOB_OFFER_RGPD_ENABLED = getattr(settings, 'JOB_OFFER_RGPD_ENABLED', False)


class ApplicationForm(forms.ModelForm):
    """Form for job application"""
    def __init__(self, *args, **kwargs):
        super(ApplicationForm, self).__init__(*args, **kwargs)

        if JOB_OFFER_RGPD_ENABLED:
            self.fields['rgpd'].label = _(
                'By submitting this form, I do agree that all '
                'information provided can be used to qualify my '
                'request and to provide updates on my application.'
            )

    class Meta:
        model = Application
        fields = [
            'message',
            'last_name',
            'first_name',
            'email',
            'phone',
            'cv',
            'cover_letter',
            'other_document',
        ]
        if JOB_OFFER_RGPD_ENABLED:
            fields.append('rgpd')


class RelatedObjectFields(type(forms.Form)):
    def __new__(self, name, bases, attrs):
        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            attrs[slug] = forms.ModelChoiceField(
                label=related_name,
                queryset=RelatedObject.objects.filter(type=slug),
                to_field_name='slug',
                required=False
            )
        return super(RelatedObjectFields, self).__new__(self, name, bases, attrs)


class SearchForm(with_metaclass(RelatedObjectFields, forms.Form)):
    keywords = forms.CharField(
        label=_('Keywords'),
        max_length=100,
        required=False
    )
    order = forms.ChoiceField(
        choices=RelatedObject._meta.get_field('type').choices,
        required=False,
    )
    reverse = forms.BooleanField(
        label=_('Reverse order'),
        required=False,
    )

    def clean_keywords(self):
        return self.cleaned_data['keywords'].split()

    def get_keywords_predicate(self, keywords):
        predicate = Q()
        for kw in keywords:
            predicate &= self.get_keyword_predicate(kw)
        return predicate

    def get_keyword_predicate(self, word):
        return (Q(title__icontains=word) |
                Q(job_description__icontains=word) |
                Q(reference__icontains=word) |
                Q(tags__icontains=word) |
                Q(competences__icontains=word))

    def filter_queryset(self, queryset):
        if self.cleaned_data.get('keywords'):
            queryset = queryset.filter(self.get_keywords_predicate(self.cleaned_data['keywords']))

        for related_object in self.cleaned_data.values():
            if not isinstance(related_object, RelatedObject):
                continue
            queryset = queryset.filter(relations=related_object)

        if not self.cleaned_data['order']:
            return queryset

        queryset = queryset.annotate(_order=functions.Coalesce(expressions.RawSQL(
            '''SELECT job_offers_relatedobject.weight
            FROM job_offers_joboffer_relations
            LEFT JOIN job_offers_relatedobject
            ON job_offers_joboffer_relations.relatedobject_id = job_offers_relatedobject.id
            WHERE joboffer_id = job_offers_joboffer.id
            AND job_offers_relatedobject.type = %s''',
            [self.cleaned_data['order']]
        ), 0))
        field = F('_order')
        return queryset.order_by(field.asc() if self.cleaned_data.get('reverse') else field.desc())
